/**
  * @file pwm_driver.c
  * @brief This module contains the implementation of the pwm driver hardware
  * function

  * @author Stefan Popovic
  *
  * @date 7/12/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: FT900
  * Dev.Board: Easyft90x v7
  * Oscillator: 100 MHz
  * Ext. Modules: PWM driver click Board
  * SW: MikroC PRO for FT90x v2.0.0

  * NOTES:
    - Place PWM driver click board on mikroBUS slot 1
    - Turn on TFT backlight on SW5.8
    - To simulate analog input on ADC channel 1, use on-board potentiometer P1
      by connecting jumper J81 (board specific) to GP06 (ADC channel 1 input)
    - Turn on LEDs on port GPIO_56_66 on SW8.8
    - PWM: GP56  _PWM_CHANNEL_0
    - ADC: GP06  CH1
    - Output LED: GP56

  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/

  /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/

  /**************************************************************************
  * Module Typedefs
  **************************************************************************/

  /**************************************************************************
  * Includes
  **************************************************************************/
  #include "resources.h"
  #include "built_in.h"
  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/
  sbit Chip_Select at GPIO_PIN28_bit;
  sbit Chip_Select1 at  GPIO_PIN9_bit;
  

  void Auxiliary_Code(void);
  void Set_Index(unsigned short index);
  void Write_Command(unsigned short cmd);
  void Write_Data(unsigned int _data);

  // TFT Init
  char TFT_DataPort at GPIO_PORT_32_39;
  sbit TFT_RST at GPIO_PIN42_bit;
  sbit TFT_RS at GPIO_PIN40_bit;
  sbit TFT_CS at GPIO_PIN41_bit;
  sbit TFT_RD at GPIO_PIN46_bit;
  sbit TFT_WR at GPIO_PIN47_bit;
  sbit TFT_BLED at GPIO_PIN58_bit;
  
  // TFT Support
  void Auxiliary_Code()
  {
   TFT_WR = 0;
   asm nop;
   TFT_WR = 1;
  }

  void Set_Index(unsigned short index) 
  {
   TFT_RS = 0;
   asm nop;
   TFT_DataPort = index;
   asm nop; asm nop;
   Auxiliary_Code();
  }

  void Write_Command(unsigned short cmd) 
  {
   TFT_RS = 1;
   asm nop;
   TFT_DataPort = cmd;
   asm nop; asm nop;
   Auxiliary_Code();
  }

  void Write_Data(unsigned int _data) 
  {
   TFT_RS = 1;
   asm nop;
   TFT_DataPort = Hi(_data);
   asm nop; asm nop;
   Auxiliary_Code();
   asm nop;
   TFT_DataPort = Lo(_data);
   asm nop; asm nop;
   Auxiliary_Code();
  }
  
  unsigned int pwm_period;
  unsigned int adc_rd;
  char txt[12];
  
  // MCU Initialization
  static void MCU_Init()
  {
    pwm_period = PWM_Init(250000);
    PWM_ConfigChannel(pwm_period , _PWM_INIT_STATE_1, _PWM_CHANNEL_0);   // PWM output on P56 pin
    PWM_EnableOutput();
  }
  
  // Display Initialization
  static void Display_Init()
  {
    TFT_Set_Active(Set_Index, Write_Command, Write_Data);
    TFT_Init_ILI9341_8bit(320, 240);
    TFT_BLED = 1;
    TFT_Set_Pen(CL_WHITE, 1);
    TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
    TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
    TFT_Fill_Screen(CL_WHITE);
    TFT_Set_Pen(CL_Black, 1);
    TFT_Line(20, 220, 300, 220);
    TFT_LIne(20,  46, 300,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("PWM driver click", 90, 14);
    TFT_Write_Text("Duty cycle:", 40, 100);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("%", 200, 100);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyFT90x v7", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);
    TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  }

  void main()
  {
     MCU_Init();
     Display_Init();

     while(1)     // Playing with Potentiometer P1, you can control current PWM duty cycle
                  //For visual result, look at LED GP56
     {
       adc_rd = ADC_Read(1);
       Delay_ms(1);
       pwm_period = adc_rd / 2 ;
       PWM_ConfigChannel(pwm_period, _PWM_INIT_STATE_1, _PWM_CHANNEL_0);
       TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL);
       IntToStr( adc_rd / 10.23, txt);
       TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
       TFT_Set_Pen(CL_WHITE, 1);
       TFT_Rectangle(145, 100, 194, 125);
       TFT_Write_Text(txt, 150, 100);
       TFT_Set_Pen(CL_BLACK, 1);
       Delay_ms(20);
     }
  }